<?php
namespace App\Controller;

use App\Entity\Person;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
#use FOS\RestBundle\Controller\FOSRestController;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\View\ViewHandler;
use FOS\RestBundle\View\View;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\ParamConverter;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;

class PersonController extends Controller
{
    /**
     * @Rest\Post(
     *     path = "/api/person/inscription",
     *     name = "create_person"
     * )
     * @Rest\View(StatusCode = Response::HTTP_CREATED)
     * @ParamConverter("person", converter="fos_rest.request_body")
     */
    public function createPersonAction(Person $person)
    {
        $em = $this->getDoctrine()->getManager();

        $em->persist($person);
        $em->flush();

        //TODO: use ViewHandler
        return $this->view($person, Response::HTTP_CREATED, ['Location' => $this->generateUrl('show_person', ['id' => $person->getId(), UrlGeneratorInterface::ABSOLUTE_URL])]);
    }

    /**
     * @Rest\Delete("/api/person/{id}")
     * @Rest\View(statusCode = Response::HTTP_NO_CONTENT)
     */
    public function removePersonAction(Request $request)
    {
        $em = $this->getDoctrine()->getManager();
        $person = $em->getRepository('App:Person')->find($request->get('id'));

        if (!empty($person)) {
            $em->remove($person);
            $em->flush();
        }
        //TODO: remove localisation and friendship
    }

    /**
     * @Rest\Put(
     *     path = "/api/person/{id}/update",
     *     name = "update_person"
     * )
     * @Rest\View(StatusCode = Response::HTTP_CREATED)
     */
    public function updatePersonAction(Request $request)
    {
        $em = $this->getDoctrine()->getManager();
        $person = $em->getRepository('App:Person')->find($request->get('id'));

        $viewHandler = $this->get('fos_rest.view_handler');

        if (empty($person)) {
            return $this->PersonNotFound($viewHandler);
        }

        $person->setFirstName($request->get('firstname'));
        $person->setLastName($request->get('lastname'));
        $person->setEmail($request->get('email'));

        $em->merge($person);
        $em->flush();

        //TODO: use ViewHandler
        return $this->view($person, Response::HTTP_CREATED, ['Location' => $this->generateUrl('show_person', ['id' => $person->getId(), UrlGeneratorInterface::ABSOLUTE_URL])]);
    }

    /**
     * @Rest\Get("/api/person/{id}/friends/localisation")
     */
    public function getFriendsLocalisationAction(Request $request)
    {

    }

    /**
     * @Rest\Get("/api/person/{id}/localisations")
     */
    public function getLocalisationsAction(Request $request)
    {
        $em = $this->getDoctrine()->getManager();
        $localisations = $em->getRepository('App:Localisation')->findBy(['person' => $request->get('id')]);

        $viewHandler = $this->get('fos_rest.view_handler');

        if (empty($localisations)) {
            return $this->PersonLocalisationsNotFound($viewHandler);
        }

        $view = View::create($localisations);
        $view->setFormat('json');

        return $viewHandler->handle($view);

    }

    /**
     * @Rest\Post("/api/person/{id}/localisation")
     * @Rest\View(StatusCode = Response::HTTP_CREATED)
     */
    public function updateLocalisationAction(Request $request)
    {
        $localisation = new Localisation();
        $localisation->setPerson($request->get('id'));
        $localisation->setTimestamp($request->get('timestamp'));
        $localisation->setLatitude($request->get('latitude'));
        $localisation->setLongitude($request->get('longitude'));

        $em = $this->getDoctrine()->getManager();

        $em->persist($localisation);
        $em->flush();
    }

    /**
    * @Rest\Get(
    *     path = "/api/person/{id}",
    *     name = "show_person",
    *     requirements = {"id"="\d+"}
    * )
    * @Rest\View()
    */
   public function showPerson(Request $request)
   {
        $em = $this->getDoctrine()->getManager();
        $person = $em->getRepository('App:Person')->find($request->get('id'));

        $viewHandler = $this->get('fos_rest.view_handler');

        if (empty($person)) {
            return $this->PersonNotFound($viewHandler);
        }

        $view = View::create($person);
        $view->setFormat('json');

        return $viewHandler->handle($view);
   }

   /**
   * @Rest\Get(
   *     path = "/api/person/{email}",
   *     name = "show_person_by_email",
   *     requirements = {"email"="\s+"}
   * )
   * @Rest\View()
   */
  public function showPersonByEmail(Request $request)
  {
        $em = $this->getDoctrine()->getManager();
        $person = $em->getRepository('App:Person')->findOneBy(['email' => $request->get('email')]);

        $viewHandler = $this->get('fos_rest.view_handler');

        if (empty($person)) {
            return $this->PersonNotFound($viewHandler);
        }

        $view = View::create($person);
        $view->setFormat('json');

        return $viewHandler->handle($view);
  }

   /**
   * @Rest\Get(
   *     path = "/api/person/{id}/friends",
   *     name = "show_person_friends",
   *     requirements = {"id"="\d+"}
   * )
   * @Rest\View()
   */
  public function showPersonFriends(Request $request)
  {
        $em = $this->getDoctrine()->getManager();
        $person = $em->getRepository('App:Person')->find($request->get('id'));

        $viewHandler = $this->get('fos_rest.view_handler');

        if (empty($person)) {
            return $this->PersonNotFound($viewHandler);
        }

        $view = View::create($person->getFriends());
        $view->setFormat('json');

        return $viewHandler->handle($view);
  }

  /**
  * @Rest\Get(
  *     path = "/api/person/{email}/friends",
  *     name = "show_person_friends_by_email",
  *     requirements = {"email"="\s+"}
  * )
  * @Rest\View()
  */
 public function showPersonFriendsByEmail(Request $request)
 {
        $em = $this->getDoctrine()->getManager();
        $person = $em->getRepository('App:Person')->findOneBy(['email' => $request->get('email')]);

        $viewHandler = $this->get('fos_rest.view_handler');

        if (empty($person)) {
            return $this->PersonNotFound($viewHandler);
        }

        $view = View::create($person->getFriends());
        $view->setFormat('json');

        return $viewHandler->handle($view);
 }

 private function PersonNotFound($viewHandler) {
     return $viewHandler->handle(View::create(['message' => 'Person not found'], Response::HTTP_NOT_FOUND));
 }

 private function PersonLocalisationsNotFound($viewHandler) {
     return $viewHandler->handle(View::create(['message' => 'Person localisations not found'], Response::HTTP_NOT_FOUND));
 }

}
