<?php
namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity()
 * @ORM\Table(name="Person", indexes={@ORM\Index(name="authentification_idx", columns={"email", "password"}),
 *                                    @ORM\Index(name="search_idx", columns={"firstname", "lastname", "email"})});
 */
class Person
{
    /**
     * @ORM\Id
     * @ORM\Column(type="bigint")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(type="string")
     */
    protected $firstname;

    /**
     * @ORM\Column(type="string")
     */
    protected $lastname;

    /**
     * @ORM\Column(type="string", unique=true)
     */
    protected $email;

    /**
     * @ORM\Column(type="string")
     */
    protected $password;

    /**
     * @ORM\Column(type="boolean", options={"default":false})
     */
    protected $online;

    /**
    * One person have many friends
    * @ORM\OneToMany(targetEntity="App\Entity\Friendship", mappedBy="person", cascade={"all"})
    */
    protected $friends;

    /**
    * One person have many friends
    * @ORM\OneToMany(targetEntity="App\Entity\Friendship", mappedBy="friend", cascade={"all"})
    */
    protected $friends_with_me;

    /**
     * [__construct description]
     * NOTE: Should we forbid reverse pairs (person_id=123 and friend_id=456
     *       can be viewed as the same as person_id=456 and friend_id=123)
     *       if we view friendship as a bidirectionnal association?
     */
    public function __construct()
    {
        $this->friends = new ArrayCollection();
        $this->friends_with_me = new ArrayCollection();
    }

    public function getId()
    {
        return $this->id;
    }

    public function getFirstName()
    {
        return $this->firstname;
    }

    public function getLastName()
    {
        return $this->lastname;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getPassword()
    {
        return $this->password;
    }

    public function getOnline()
    {
        return $this->online;
    }

    public function getFriends()
    {
        return $this->friends;
    }

    public function getFriendsWithMe()
    {
        return $this->friends_with_me;
    }

    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }

    public function setFirstName($firstname)
    {
        $this->firstname = $firstname;
        return $this;
    }

    public function setLastName($lastname)
    {
        $this->lastname = $lastname;
        return $this;
    }

    public function setEmail($email)
    {
        $this->email = $email;
        return $this;
    }

    public function setPassword($password)
    {
        $this->password = $password;
        return $this;
    }

    public function setOnline($online)
    {
        $this->online = $online;
        return $this;
    }

    public function addFriendship(Friendship $friendship)
    {
        $this->friends->add($friendship);
        $friendship->friend->addFriendshipWithMe($friendship);
    }

    public function addFriendshipWithMe(Friendship $friendship)
    {
        $this->friends_with_me->add($friendship);
    }

    public function addFriend(Person $friend)
    {
        $fs = new Friendship();
        $fs->setPerson($this);
        $fs->setFriend($friend);
        // set defaults validation to true for now
        $fs->setIsValid(true);

        $this->addFriendship($fs);
    }
}
